class Comment:

    def __init__(self, filepath: str, text: str, line_num: int) -> None:
        self.filepath = filepath
        self.text = text
        self.line_num = line_num


class File:

    def __init__(self, path: str) -> None:
        self.path = path


class FileExtension:

    SUPPORTED = [
        'go',
        'js',
        'php',
        'py',
        'rb'
    ]



from .exceptions import LanguageUnsupported
from .parser import Directory as DirectoryParser
from .validator import Comment as Validator


def evaluate(path: str) -> None:
    parser = DirectoryParser(path)

    for _file in parser:
        if not _file.is_supported():
            continue

        for comment in _file:
            try:
                validate = Validator(comment)
                assert bool(validate)
            except AssertionError:
                raise LanguageUnsupported.from_validator(validate)



import io
import os
from typing import Iterable

from pygments.lexers import get_lexer_for_filename
from pygments.token import Token
import pygments.util

from .models import Comment, File, FileExtension


class Directory:

    def __init__(self, path: str) -> None:
        self.path = path

    def filepaths(self, path=None) -> Iterable[File]:
        """Yields File recursively from this directory."""
        if not path:
            path = self.path
        for dirpath, dirs, files in os.walk(path):
            for name in files:
                    filename = os.path.join(dirpath, name)
                    yield File(os.path.abspath(filename))
                    if not dirs:
                        continue
                    else:
                        for d in dirs:
                            self.filepaths(os.path.join(dirpath, d))

    def __iter__(self) -> Iterable[File]:
        return self.filepaths()


class File:

    def __init__(self, path: str) -> None:
        self.path = path
        try:
            self.lexer = get_lexer_for_filename(path)
        except pygments.util.ClassNotFound:
            self.lexer = None

    def is_supported(self) -> bool:
        filename, ext = os.path.splitext(self.path)
        return ext[1:] in FileExtension.SUPPORTED

    def comments(self, path=None) -> Iterable[Comment]:
        """Yields Comments in this file."""
        if not self.lexer:
            return

        with io.open(self.path, 'r', encoding='utf8') as f:
            try:
                for (idx, Class, value) in self.lexer.get_tokens_unprocessed(
                        f.read()
                ):
                    if Class in (
                            Token.Comment.Multiline,
                            Token.Comment.Single
                    ):
                        yield Comment(self.path, value, idx)

            except UnicodeDecodeError:
                pass

    def __iter__(self) -> Iterable[Comment]:
        return self.comments()



from langdetect import detect_langs
from langdetect.lang_detect_exception import LangDetectException

from .models import Comment


class Comment:
    MIN_DETECTION_CONFIDENCE = 0.90

    def __init__(self, comment: Comment, language_code: str='en') -> None:
        self.comment = comment
        self.language_code = language_code
        self.language_code_detected = None

    def is_valid(self) -> bool:
        try:
            self.validate()
        except AssertionError:
            return False
        else:
            return True

    def validate(self) -> None:
        try:
            guess = detect_langs(self.comment.text)[0]
        except (IndexError, LangDetectException):
            # skip validation since we cannot guess the language code
            return
        else:
            self.language_code_detected = guess.lang
            if float(guess.prob) < self.MIN_DETECTION_CONFIDENCE:
                return

            assert self.language_code_detected == self.language_code

    def __bool__(self) -> bool:
        return self.is_valid()



from .validator import Comment as CommentValidator


class LanguageUnsupported(BaseException):

    @classmethod
    def from_validator(cls, validator: CommentValidator):
        msg = (
            'Comment "{text}" detected to be \'{detected}\','
            'not in \'{language_code}\':'
            'at: {path}:{line_num}'.format(
                text=validator.comment.text,
                detected=validator.language_code_detected,
                language_code=validator.language_code,
                path=validator.comment.filepath,
                line_num=validator.comment.line_num
            )
        )
        return cls(msg)



import os
import sys

import click

from .service import evaluate


@click.command('')
@click.option('--path', default='.', help='path to project or repository')
def main(path: str):
    """Simple tool to detect non-English commments in a code base."""
    try:
        fpath = full_path(path)
    except AssertionError:
        raise click.BadParameter(
            'path specified needs to be a directory',
            param_hint=['--path']
        )
    try:
        evaluate(fpath)
    except AssertionError:
        # TODO: error indicator
        sys.exit(1)
    else:
        # TODO: success indicator
        sys.exit(0)


def full_path(path: str):
    if path == '.':
        return os.getcwd()
    assert os.path.isdir(path)
    if os.path.isabs(path):
        return path
    return os.path.abspath(path)


if __name__ == '__main__':
    main()



import io
import re
from setuptools import setup


with io.open('README.md', 'rt', encoding='utf8') as f:
    readme = f.read()

with io.open('pulpfiction/__init__.py', 'rt', encoding='utf8') as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

setup(
    name='pulpfiction',
    version=version,
    url='https://github.com/kelvintaywl/pulpfiction',
    author='Kelvin Tay',
    author_email='kelvintaywl@gmail.com',
    maintainer='Kelvin Tay',
    maintainer_email='kelvintaywl@gmail.com',
    description='A simple utility tool to detect non-English comments in code',
    long_description=readme,
    packages=['pulpfiction'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'pygments>=2.2.0',
        'langdetect>=1.0.7',
        'click>=6.7',
    ],
    extras_require={
        'dev': [
            'flake8>=3.5.0'
        ]
    },
    python_requires='>=3',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        'console_scripts': [
            'jules = pulpfiction.cli:main',
        ],
    },
)
